<?php

namespace Drupal\acquia_contenthub\EventSubscriber\EntityDataTamper;

use Drupal\acquia_contenthub\AcquiaContentHubEvents;
use Drupal\acquia_contenthub\Event\EntityDataTamperEvent;
use Drupal\acquia_contenthub\Event\ParseCdfEntityEvent;
use Drupal\depcalc\DependentEntityWrapper;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Replace recognized remote user with an appropriate local user.
 */
class ExistingUser implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[AcquiaContentHubEvents::ENTITY_DATA_TAMPER][] = 'onDataTamper';
    $events[AcquiaContentHubEvents::PARSE_CDF][] = ['onParseCDF', 90];

    return $events;
  }

  /**
   * Tamper with CDF data before its imported.
   *
   * @param \Drupal\acquia_contenthub\Event\EntityDataTamperEvent $event
   *   The data tamper event.
   *
   * @throws \Exception
   */
  public function onDataTamper(EntityDataTamperEvent $event) {
    $cdf = $event->getCDF();

    /** @var \Acquia\ContentHubClient\CDF\CDFObject $object */
    foreach ($cdf->getEntities() as $uuid => $object) {
      $entity_type = $object->getAttribute('entity_type');
      if (!$entity_type || $entity_type->getValue()['und'] !== 'user') {
        continue;
      }

      $account = $object->getAttribute('mail')
        ? user_load_by_mail($object->getAttribute('mail')->getValue()['und'])
        : user_load_by_name($object->getAttribute('username')->getValue()['und']);
      if (!$account || (!$object->getAttribute('mail') && '' !== $account->mail)) {
        continue;
      }

      /** @var \Drupal\Core\Entity\EntityInterface $account */
      $wrapper = new DependentEntityWrapper($account);
      $wrapper->setRemoteUuid($uuid);
      $event->getStack()->addDependency($wrapper);
    }
  }

  /**
   * Parses the CDF representation of Content Entities.
   *
   * @throws \Exception
   */
  public function onParseCDF(ParseCdfEntityEvent $event) {
    $cdf = $event->getCDF();
    if ($cdf->getAttribute('entity_type')->getValue()['und'] !== 'user') {
      // Bail early if this isn't a content entity.
      return;
    }

    $uuid = $cdf->getUuid();
    $account = \Drupal::entityTypeManager()->getStorage('user')->loadByProperties(['uuid' => $uuid]);
    if ($account) {
      return;
    }

    $username = $cdf->getAttribute('username')->getValue()['und'];
    if ($account = user_load_by_name($username)) {
      /** @var \Drupal\user\Entity\User $entity */
      $entity = $event->getEntity();
      $entity->setUsername($cdf->getUuid() . ' (' . $username . ')');
      $event->setEntity($entity);
    }
  }

}
